<?php
/**
 * Perfect Checkout - Admin Menus.
 *
 * @author  Sébastien Dumont
 * @package Perfect Checkout\Admin\Menus
 * @since   1.0.0
 * @version 1.4.0
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'Perfect_Checkout_Admin_Menus' ) ) {

	class Perfect_Checkout_Admin_Menus {

		public static $title = 'Perfect Checkout';

		public static $menu_title;

		public static $breadcrumb;

		/**
		 * Constructor
		 *
		 * @access public
		 */
		public function __construct() {
			self::$menu_title = self::$title;
			self::$breadcrumb = self::$title;

			add_action( 'admin_menu', array( $this, 'settings_menu' ) );
			add_action( 'admin_menu', array( $this, 'wc_admin_bar' ) );
			add_action( 'admin_menu', array( $this, 'wc_admin_navigation' ) );

			// Handle saving settings earlier than load-{page} hook to avoid race conditions in conditional menus.
			add_action( 'wp_loaded', array( $this, 'save_settings' ) );

			// Handles disconnecting Perfect Checkout.
			add_action( 'wp_loaded', array( $this, 'disconnect_perfectcheckout' ) );
		} // END __construct()

		/**
		 * Registers the WooCommerce Admin bar.
		 *
		 * @access public
		 */
		public function wc_admin_bar() {
			if ( Perfect_Checkout_Helpers::is_wc_version_gte( '4.0' ) && function_exists( 'wc_admin_connect_page' ) ) {
				wc_admin_connect_page(
					array(
						'id'        => 'perfect-checkout-settings',
						'screen_id' => 'toplevel_page_perfect-checkout',
						'title'     => array(
							self::$title,
							self::$breadcrumb,
						),
						'path'      => add_query_arg(
							array(
								'page' => 'perfect-checkout',
							),
							'admin.php'
						),
					)
				);
			}
		} // END wc_admin_bar()

		/**
		 * Registers the WooCommerce Navigation.
		 *
		 * @access public
		 * @since 1.0.7
		 */
		public function wc_admin_navigation() {
			// Adds Perfect Checkout page to the new WooCommerce Navigation Menu.
			if ( class_exists( '\Automattic\WooCommerce\Admin\Features\Navigation\Menu' ) ) {
				Automattic\WooCommerce\Admin\Features\Navigation\Menu::add_plugin_item(
					array(
						'id'         => 'perfect-checkout',
						'title'      => self::$title,
						'capability' => apply_filters( 'perfect_checkout_screen_capability', 'manage_options' ),
						'url'        => 'perfect-checkout',
					)
				);
			}
		} // END wc_admin_navigation()

		/**
		 * Add Perfect Checkout to the menu.
		 *
		 * @access public
		 */
		public function settings_menu() {
			$settings_page = add_menu_page(
				self::$title,
				self::$menu_title,
				apply_filters( 'perfect_checkout_screen_capability', 'manage_options' ),
				'perfect-checkout',
				array( $this, 'settings_page' ),
				untrailingslashit( plugins_url( '/', PERFECTCHECKOUT_FILE ) ) . '/assets/images/pc-tick-x16.svg'
			);

			add_action( 'load-' . $settings_page, array( $this, 'settings_page_init' ) );
		} // END settings_menu()

		/**
		 * Perfect Checkout Page
		 *
		 * @access public
		 * @static
		 */
		public static function settings_page_init() {
			// We should only save on the settings page.
			if ( ! is_admin() || ! isset( $_GET['page'] ) || 'perfect-checkout' !== $_GET['page'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended
				return;
			}

			Perfect_Checkout_Admin_Settings::get_settings_pages();

			// Add any posted messages.
			if ( ! empty( $_GET['perfect_checkout_error'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended
				Perfect_Checkout_Admin_Settings::add_error( wp_kses_post( wp_unslash( $_GET['perfect_checkout_error'] ) ) ); // phpcs:ignore WordPress.Security.NonceVerification.Recommended
			}

			if ( ! empty( $_GET['perfect_checkout_message'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended
				Perfect_Checkout_Admin_Settings::add_message( wp_kses_post( wp_unslash( $_GET['perfect_checkout_message'] ) ) ); // phpcs:ignore WordPress.Security.NonceVerification.Recommended
			}

			do_action( 'perfect_checkout_settings_page_init' );
		} // END settings_page_init()

		/**
		 * Handle saving of settings.
		 *
		 * @access public
		 * @static
		 * @global string $current_view
		 */
		public static function save_settings() {
			global $current_view;

			// We should only save on the settings page.
			if ( ! is_admin() || ! isset( $_GET['page'] ) || 'perfect-checkout' !== $_GET['page'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended
				return;
			}

			Perfect_Checkout_Admin_Settings::get_settings_pages();

			// Get current view.
			$current_view = empty( $_GET['view'] ) ? 'general' : sanitize_title( wp_unslash( $_GET['view'] ) ); // phpcs:ignore WordPress.Security.NonceVerification.Recommended

			// Save settings if data has been posted.
			if ( ! empty( $_POST['save'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
				Perfect_Checkout_Admin_Settings::save();
			}
		} // END save_settings()

		/**
		 * Initialize the Perfect Checkout settings page.
		 *
		 * @access public
		 */
		public function settings_page() {
			include_once dirname( __FILE__ ) . '/class-perfect-checkout-admin-settings.php';

			Perfect_Checkout_Admin_Settings::output();
		} // END settings_page()

		/**
		 * Handles disconnecting Perfect Checkout.
		 *
		 * @access public
		 * @since  1.4.0
		 */
		public function disconnect_perfectcheckout() {
			if ( isset( $_REQUEST['pc-disconnect'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_REQUEST['_wpnonce'] ) ), 'disconnect-perfectcheckout' ) ) {
				// Disconnect.
				wp_remote_post( add_query_arg( array(
					'lic' => get_perfect_license()
				), esc_url( PerfectCheckout::get_api_url() . '/api/webhook/disconnect-store' ) ) );

				delete_option( 'perfectcheckout_license' );

				// Revokes all WooCommerce API Keys associated with Perfect Checkout.
				Perfect_Checkout_Admin::revokes_all_pc_wc_api_keys();

				// Redirect back to settings page fresh.
				wp_safe_redirect( add_query_arg( array(
					'page' => 'perfect-checkout',
				), admin_url( 'admin.php' ) ) );
			}
		} // END disconnect_perfectcheckout()

	} // END class

} // END if class exists

return new Perfect_Checkout_Admin_Menus();
