<?php
/**
 * Display notices in the WordPress admin for Perfect Checkout.
 *
 * Forked the notice system from: https://github.com/woocommerce/woocommerce/blob/master/includes/admin/class-wc-admin-notices.php
 *
 * @author  Sébastien Dumont
 * @since   1.6.0
 * @package Perfect Checkout\Admin\Notices
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Perfect_Checkout_Admin_Notices {

	/**
	 * Stores notices.
	 *
	 * @access public
	 *
	 * @static
	 *
	 * @var array
	 */
	public static $notices = array();

	/**
	 * Array of notices - name => callback.
	 *
	 * @access private
	 *
	 * @static
	 *
	 * @var array
	 */
	private static $core_notices = array(
		'check_wc' => 'check_woocommerce_notice',
	);

	/**
	 * Constructor
	 *
	 * @access public
	 */
	public function __construct() {
		self::$notices = get_option( 'perfectcheckout_admin_notices', array() );

		add_action( 'switch_theme', array( $this, 'reset_admin_notices' ) );
		add_action( 'perfectcheckout_installed', array( $this, 'reset_admin_notices' ) );
		add_action( 'wp_loaded', array( $this, 'hide_notices' ) );

		add_action( 'shutdown', array( $this, 'store_notices' ) );

		// If the current user has capabilities then add notices.
		if ( Perfect_Checkout_Helpers::user_has_capabilities() ) {
			add_action( 'admin_print_styles', array( $this, 'add_notices' ) );
		}
	} // END __construct()

	/**
	 * Store notices to DB.
	 *
	 * @access public
	 *
	 * @static
	 */
	public static function store_notices() {
		update_option( 'perfectcheckout_admin_notices', self::get_notices() );
	} // END store_notices()

	/**
	 * Get notices
	 *
	 * @access public
	 *
	 * @static
	 *
	 * @return array
	 */
	public static function get_notices() {
		return self::$notices;
	} // END get_notices()

	/**
	 * Remove all notices.
	 *
	 * @access public
	 *
	 * @static
	 */
	public static function remove_all_notices() {
		self::$notices = array();
	} // END remove_all_notices()

	/**
	 * Reset notices for when new version of Perfect Checkout is installed.
	 *
	 * @access public
	 */
	public function reset_admin_notices() {
		self::add_notice( 'check_wc' );
	} // END reset_admin_notices()

	/**
	 * Show a notice.
	 *
	 * @access public
	 *
	 * @static
	 *
	 * @param string $name Notice name.
	 * @param bool   $force_save Force saving inside this method instead of at the 'shutdown'.
	 */
	public static function add_notice( $name, $force_save = false ) {
		self::$notices = array_unique( array_merge( self::get_notices(), array( $name ) ) );

		if ( $force_save ) {
			// Adding early save to prevent more race conditions with notices.
			self::store_notices();
		}
	} // END add_notice()

	/**
	 * Remove a notice from being displayed.
	 *
	 * @access public
	 *
	 * @static
	 *
	 * @param string $name Notice name.
	 * @param bool   $force_save Force saving inside this method instead of at the 'shutdown'.
	 */
	public static function remove_notice( $name, $force_save = false ) {
		$notices = self::get_notices();

		// Check that the notice exists before attempting to remove it.
		if ( in_array( $name, $notices ) ) {
			self::$notices = array_diff( $notices, array( $name ) );

			delete_option( 'perfectcheckout_admin_notice_' . $name );

			if ( $force_save ) {
				// Adding early save to prevent more conditions with notices.
				self::store_notices();
			}
		}
	} // END remove_notice()

	/**
	 * See if a notice is being shown.
	 *
	 * @access public
	 *
	 * @static
	 *
	 * @param string $name Notice name.
	 *
	 * @return boolean
	 */
	public static function has_notice( $name ) {
		return in_array( $name, self::get_notices(), true );
	} // END has_notice()

	/**
	 * Hide a notice if the GET variable is set.
	 *
	 * @access public
	 */
	public function hide_notices() {
		if ( isset( $_GET['perfectcheckout-hide-notice'] ) && isset( $_GET['_perfectcheckout_notice_nonce'] ) ) {
			if ( ! wp_verify_nonce( sanitize_key( wp_unslash( $_GET['_perfectcheckout_notice_nonce'] ) ), 'perfectcheckout_hide_notices_nonce' ) ) {
				wp_die( esc_html__( 'Action failed. Please refresh the page and retry.', 'perfect-checkout' ) );
			}

			if ( ! Perfect_Checkout_Helpers::user_has_capabilities() ) {
				wp_die( esc_html__( 'You don&#8217;t have permission to do this.', 'perfect-checkout' ) );
			}

			$hide_notice = sanitize_text_field( wp_unslash( $_GET['perfectcheckout-hide-notice'] ) );

			self::remove_notice( $hide_notice );

			update_user_meta( get_current_user_id(), 'dismissed_perfectcheckout_' . $hide_notice . '_notice', true );

			do_action( 'perfectcheckout_hide_' . $hide_notice . '_notice' );

			wp_safe_redirect( remove_query_arg( array( 'perfectcheckout-hide-notice', '_perfectcheckout_notice_nonce' ), Perfect_Checkout_Helpers::perfect_checkout_get_current_admin_url() ) );
			exit;
		}
	} // END hide_notices()

	/**
	 * Add notices.
	 *
	 * @access public
	 */
	public function add_notices() {
		// Prevent notices from loading on the frontend.
		if ( ! is_admin() ) {
			return;
		}

		$notices = self::get_notices();

		if ( empty( $notices ) ) {
			return;
		}

		// Notice should only show on specific pages.
		if ( ! Perfect_Checkout_Helpers::is_perfect_checkout_admin_page() ) {
			return;
		}

		foreach ( $notices as $notice ) {
			if ( ! empty( self::$core_notices[ $notice ] ) && apply_filters( 'perfectcheckout_show_admin_notice', true, $notice ) ) {
				add_action( 'admin_notices', array( $this, self::$core_notices[ $notice ] ) );
			} else {
				add_action( 'admin_notices', array( $this, 'output_custom_notices' ) );
			}
		}
	} // END add_notices()

	/**
	 * Add a custom notice.
	 *
	 * @access public
	 *
	 * @static
	 *
	 * @param string $name        Notice name.
	 * @param string $notice_html Notice HTML.
	 */
	public static function add_custom_notice( $name, $notice_html ) {
		self::add_notice( $name );
		update_option( 'perfectcheckout_admin_notice_' . $name, wp_kses_post( $notice_html ) );
	} // END add_custom_notice()

	/**
	 * Output any stored custom notices.
	 *
	 * @access public
	 *
	 * @return void
	 */
	public function output_custom_notices() {
		$notices = self::get_notices();

		if ( ! empty( $notices ) ) {
			foreach ( $notices as $notice ) {
				if ( empty( self::$core_notices[ $notice ] ) ) {
					$notice_html = get_option( 'perfectcheckout_admin_notice_' . $notice );

					if ( $notice_html ) {
						include dirname( __FILE__ ) . '/views/html-notice-custom.php';
					}
				}
			}
		}
	} // END output_custom_notices()

	/**
	 * Check WooCommerce Dependency.
	 *
	 * @access public
	 *
	 * @return void
	 */
	public function check_woocommerce_notice() {
		if ( ! defined( 'WC_VERSION' ) ) {
			// WooCommerce is Not Installed or Activated Notice.
			include_once dirname( __FILE__ ) . '/views/html-notice-wc-not-installed.php';
		} elseif ( version_compare( WC_VERSION, PerfectCheckout::$required_woo, '<' ) ) {
			/**
			 * Displays a warning message if minimum version of WooCommerce check fails and
			 * provides an update button if the user has admin capabilities to update the plugin.
			 */
			include_once dirname( __FILE__ ) . '/views/html-notice-required-wc.php';
		}
	} // END check_woocommerce_notice()

} // END class.

return new Perfect_Checkout_Admin_Notices();
