<?php
/**
 * Perfect Checkout - Admin.
 *
 * @author  Sébastien Dumont
 * @package Perfect Checkout\Admin
 * @since   1.0.0
 * @version 1.6.11
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'Perfect_Checkout_Admin' ) ) {

	class Perfect_Checkout_Admin {

		/**
		 * Constructor
		 *
		 * @access  public
		 * @since   1.0.0
		 * @version 1.6.0
		 */
		public function __construct() {
			add_action( 'init', array( $this, 'includes' ) );
			add_action( 'current_screen', array( $this, 'conditional_includes' ) );

			add_action( 'admin_notices', array( $this, 'connect_perfectcheckout_notice' ) );
			add_action( 'admin_notices', array( $this, 'remind_user_activate_pc_again' ) );
			add_action( 'admin_notices', array( $this, 'activate_perfect_again' ) );

			// Runs when the plugin is activated to check the environment.
			register_activation_hook( PERFECTCHECKOUT_FILE, array( $this, 'check_environment' ) );

			// Removes the Perfect Checkout page and saves a reminder.
			register_deactivation_hook( PERFECTCHECKOUT_FILE, array( $this, 'deactivate_pc' ) );

			// Adds new field types for WooCommerce settings.
			add_action( 'woocommerce_admin_field_pc_button', array( $this, 'pc_button_field' ) );

			// Hides the option "Hide shipping costs until an address is entered".
			add_filter( 'woocommerce_shipping_settings', array( $this, 'hide_shipping_option' ), 0 );

			// Takes control of the page settings to support switching over to PC.
			add_filter( 'woocommerce_settings_pages', array( $this, 'woocommerce_settings_pages' ) );

			// Add a post display state for "Perfect Checkout" page.
			add_filter( 'display_post_states', array( $this, 'add_display_post_states' ), 11, 2 );

			// Removes the "Edit" and "Trash" link from the page action links.
			add_filter( 'page_row_actions', array( $this, 'remove_page_action_links' ), 10, 2 );

			// Handles disconnecting Perfect Checkout.
			add_action( 'wp_loaded', array( $this, 'setup_perfect_checkout_page' ) );

			// Verifies the Perfect Checkout page is setup.
			add_action( 'wp_loaded', array( $this, 'verify_perfect_checkout_page' ), 20 );
		} // END __construct()

		/**
		 * Displays an admin notice to remind the user
		 * to complete the connection for Perfect Checkout.
		 *
		 * @access public
		 * @since  1.5.0
		 */
		public function connect_perfectcheckout_notice() {
			$screen    = get_current_screen();
			$screen_id = $screen ? $screen->id : '';

			if ( 'toplevel_page_perfect-checkout' !== $screen_id ) {
				if ( empty( get_option( 'perfectcheckout_hide_activation_notice' ) ) ) {
					include_once dirname( PERFECTCHECKOUT_FILE ) . '/includes/admin/views/html-admin-notice-connect.php';

					// Disable activation notice.
					add_option( 'perfectcheckout_hide_activation_notice', true );
				}
			}
		} // END connect_perfectcheckout_notice()

		/**
		 * Displays an admin notice to remind the user
		 * to reactivate Perfect Checkout when ready.
		 *
		 * @access public
		 * @since  1.5.0
		 */
		public function remind_user_activate_pc_again() {
			if ( ! empty( get_option( 'perfectcheckout_remind_user_pc_was_active' ) ) ) {
				include_once dirname( PERFECTCHECKOUT_FILE ) . '/includes/admin/views/html-admin-notice-reminder.php';

				// Clears activation notice on next page load.
				delete_option( 'perfectcheckout_remind_user_pc_was_active' );
			}
		} // END remind_user_activate_pc_again()

		/**
		 * Displays an admin notice to notify the user
		 * Perfect Checkout needs activating again.
		 *
		 * @access public
		 * @since  1.5.0
		 */
		public function activate_perfect_again() {
			if ( ! empty( get_option( 'perfectcheckout_incomplete' ) ) ) {
				include_once dirname( PERFECTCHECKOUT_FILE ) . '/includes/admin/views/html-admin-notice-incomplete.php';

				// Clears notice on next page load.
				delete_option( 'perfectcheckout_incomplete' );
			} elseif ( ! empty( get_option( 'perfectcheckout_page_missing' ) ) ) {
				include_once dirname( PERFECTCHECKOUT_FILE ) . '/includes/admin/views/html-admin-notice-page-missing.php';

				// Clears notice on next page load.
				delete_option( 'perfectcheckout_page_missing' );
			}
		} // END activate_perfect_again()

		/**
		 * Checks the environment for the plugin.
		 *
		 * @access  public
		 * @since   1.1.2
		 * @version 1.6.0
		 */
		public function check_environment() {
			/**
			* Deactivate either CoCart Lite and CoCart Pro to prevent conflict
			* if either of them are a version lower than the packaged versions.
			*/
			if ( Perfect_Checkout_Helpers::is_cocart_installed() ) {
				if ( Perfect_Checkout_Helpers::is_cocart_version_lte( CoCart_Packages::get_package_version( 'CoCart' ) ) ) {
					deactivate_plugins( plugin_basename( COCART_FILE ) );
				}
			}

			if ( Perfect_Checkout_Helpers::is_cocart_pro_installed() ) {
				deactivate_plugins( plugin_basename( COCART_PRO_FILE ) );
			}

			Perfect_Checkout_Install::maybe_create_page();
		} // END check_environment()

		/**
		 * When requested via the settings page, Perfect Checkout will
		 * change the cart page based on the status requested.
		 *
		 * @access  public
		 * @since   1.4.0
		 * @version 1.5.0
		 */
		public function setup_perfect_checkout_page() {
			if ( isset( $_REQUEST['checkout'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_REQUEST['_wpnonce'] ) ), 'switch-checkout' ) ) {

				$checkout = sanitize_text_field( wp_unslash( $_REQUEST['checkout'] ) );
				$referral = isset( $_REQUEST['referral'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['referral'] ) ) : '';

				// Switching to Perfect Checkout.
				if ( 'perfect' === $checkout ) {
					// Create new page for "Perfect Checkout" if one does not yet exist.
					Perfect_Checkout_Install::maybe_create_page();

					add_option( 'perfectcheckout_set_active', 'yes' );
				}
				// Switch to default checkout.
				else {
					// Delete the "Perfect Checkout" page.
					self::maybe_delete_pc_page();

					delete_option( 'perfectcheckout_set_active' );
				}

				// Redirect back to settings page.
				if ( ! empty( $referral ) && 'wc-settings' === $referral ) {
					$settings_page = add_query_arg( array(
						'page' => 'wc-settings',
						'tab'  => 'advanced',
					), admin_url( 'admin.php' ) );
				} else {
					$settings_page = add_query_arg( array(
						'page' => 'perfect-checkout',
					), admin_url( 'admin.php' ) );
				}

				wp_safe_redirect( $settings_page );
			}
		} // END setup_perfect_checkout_page()

		/**
		 * Verifies that the Perfect Checkout page is setup.
		 *
		 * @access public
		 * @since  1.5.0
		 */
		public static function verify_perfect_checkout_page() {
			$pc_active  = get_option( 'perfectcheckout_set_active' );
			$pc_page_id = get_option( 'perfectcheckout_page_id' );

			if ( 'yes' === $pc_active && empty( $pc_page_id ) ) {
				// Deactivate Perfect Checkout.
				delete_option( 'perfectcheckout_set_active' );

				// Ask user to activate Perfect Checkout again.
				add_option( 'perfectcheckout_incomplete', 'yes' );
			} elseif ( 'yes' === $pc_active && $pc_page_id > 0 ) {
				$page = get_post( $pc_page_id, 'ARRAY_A' );

				// If the page is set to "Draft" or "Trash", automatically publish it again.
				if ( 'draft' === $page['post_status'] || 'trash' === $page['post_status'] ) {
					$page['post_status'] = 'publish';
					wp_update_post( $page );
				} else {
					if ( ! $page ) {
						// Deactivate Perfect Checkout.
						delete_option( 'perfectcheckout_set_active' );
						delete_option( 'perfectcheckout_page_id' );

						// Notify user that the page has been deleted.
						add_option( 'perfectcheckout_page_missing', 'yes' );
					}
				}
			}
		} // END verify_perfect_checkout_page()

		/**
		 * Removes our checkout page and sets a reminder.
		 *
		 * @access  public
		 * @since   1.0.2
		 * @version 1.5.0
		 */
		public function deactivate_pc( $delete_page = true ) {
			// Remove Perfect Checkout page if true.
			if ( $delete_page ) {
				self::maybe_delete_pc_page();
			}

			$pc_active = get_option( 'perfectcheckout_set_active' );

			// Set reminder if PC was active before.
			if ( ! empty( $pc_active ) ) {
				add_option( 'perfectcheckout_remind_user_pc_was_active', 'yes' );
				delete_option( 'perfectcheckout_set_active' );
			}
		} // END deactivate_pc()

		/**
		 * Delete "Perfect Checkout" page.
		 *
		 * @access  protected
		 * @since   1.0.2
		 * @version 1.5.0
		 * @return  void
		 */
		protected function maybe_delete_pc_page() {
			$pc_page_id = get_option( 'perfectcheckout_page_id' );

			if ( $pc_page_id ) {
				wp_delete_post( $pc_page_id, true );
			}

			delete_option( 'perfectcheckout_page_id' );
		} // END maybe_delete_pc_page()

		/**
		 * Include any classes we need within admin.
		 *
		 * @access public
		 */
		public function includes() {
			include_once dirname( PERFECTCHECKOUT_FILE ) . '/includes/admin/class-perfect-checkout-admin-assets.php';   // Admin Assets.
			include_once dirname( PERFECTCHECKOUT_FILE ) . '/includes/admin/class-perfect-checkout-admin-notices.php';  // Admin Notices.
			include_once dirname( PERFECTCHECKOUT_FILE ) . '/includes/admin/class-perfect-checkout-admin-settings.php'; // Admin Settings.
			include_once dirname( PERFECTCHECKOUT_FILE ) . '/includes/admin/class-perfect-checkout-admin-menus.php';    // Admin Menus.
		} // END includes()

		/**
		 * Include admin files conditionally.
		 *
		 * @access public
		 * @since  1.6.0
		 */
		public function conditional_includes() {
			$screen = get_current_screen();

			if ( ! $screen ) {
				return;
			}

			switch ( $screen->id ) {
				case 'plugins':
					include_once dirname( PERFECTCHECKOUT_FILE ) . '/includes/admin/class-perfect-checkout-admin-action-links.php';  // Action Links.
					include_once dirname( PERFECTCHECKOUT_FILE ) . '/includes/admin/class-perfect-checkout-admin-plugin-screen.php'; // Admin Plugin Screen.

					break;
			}
		} // END conditional_includes()

		/**
		 * Hides the option "Hide shipping costs until an address is entered".
		 *
		 * @access public
		 * @param  array $settings - Original settings.
		 * @return array $settings - New settings
		 */
		public function hide_shipping_option( $settings ) {
			unset( $settings[2] );

			return $settings;
		} // END hide_shipping_option()

		/**
		 * Adds a new button field with title and description for WooCommerce settings.
		 *
		 * @access public
		 * @since  1.5.0
		 * @param  array $value
		 */
		public function pc_button_field( $value ) {
			?>
			</table>
			<?php
			echo '<hr>';

			if ( ! empty( $value['title'] ) ) {
				echo '<h2>' . esc_html( $value['title'] ) . '</h2>';
			}

			if ( ! empty( $value['desc'] ) ) {
				echo '<div id="' . esc_attr( sanitize_title( $value['id'] ) ) . '-description">';
				echo wp_kses_post( wpautop( wptexturize( $value['desc'] ) ) );
				echo '</div>';
			}
			?>
			<table class="form-table">
				<tr valign="top" id="button_<?php echo esc_attr( $value['id'] ); ?>">
					<th scope="row" class="titledesc">
						<img src="<?php echo esc_url( untrailingslashit( plugins_url( '/', PERFECTCHECKOUT_FILE ) ) . '/assets/images/logo.svg' ); ?>" height="32px" alt="Perfect Checkout Logo" />
						&nbsp;&nbsp;
						<a href="<?php echo esc_html( $value['url'] ); ?>" class="button button-secondary <?php echo esc_attr( $value['class'] ); ?>">
							<?php echo esc_html( $value['value'] ); ?>
						</a>
					</th>
				</tr>
			</table>
			<hr>
			<table class="form-table">
			<?php
		} // END pc_button_field()

		/**
		 * Adds new settings field for WooCommerce page settings.
		 *
		 * @access public
		 * @since  1.5.0
		 * @param  array $settings Original settings.
		 * @return array $settings New settings
		 */
		public function woocommerce_settings_pages( $settings ) {
			$pc_license = get_perfect_license();

			// Insert before my account page settings.
			$myaccount_page = wp_list_filter( $settings, array(
				'id'   => 'woocommerce_myaccount_page_id',
				'type' => 'single_select_page_with_search',
			) );

			if ( ! empty( $pc_license ) ) {
				$pc_active = get_option( 'perfectcheckout_set_active', 'no' );

				if ( 'yes' === $pc_active ) {
					$switch_checkout_url = wp_nonce_url( add_query_arg( array(
						'referral' => 'wc-settings',
						'checkout' => 'default',
					), admin_url( 'admin.php' ) ), 'switch-checkout' );

					$new_settings = array(
						'set_wc_active' => array(
							'title'    => sprintf(
								/* translators: %s Perfect Checkout */
								esc_html__( '%s is Active', 'perfect-checkout' ),
								'Perfect Checkout'
							),
							'desc'     => sprintf(
								/* translators: %1$s Perfect Checkout, %2$s Link to customize guide, %3$s Close link */
								esc_html__( '%1$s is now active and handling your cart and checkout needs. You can manually change your checkout page above if desired. %2$sLearn how to customize these settings for your needs%3$s.', 'perfect-checkout' ),
								'Perfect Checkout',
								'<a href="#">',
								'</a>'
							),
							'id'       => '',
							'type'     => 'pc_button',
							'value'    => sprintf(
								/* translators: %s Perfect Checkout */
								esc_html__( 'Disable %s', 'perfect-checkout' ),
								'Perfect Checkout'
							),
							'url'      => $switch_checkout_url,
							'autoload' => false,
						),
					);
				} else {
					$switch_checkout_url = wp_nonce_url( add_query_arg( array(
						'referral' => 'wc-settings',
						'checkout' => 'perfect',
					), admin_url( 'admin.php' ) ), 'switch-checkout' );

					$new_settings = array(
						'set_pc_active' => array(
							'title'    => sprintf(
								/* translators: %s Perfect Checkout */
								esc_html__( 'Ready for %s?', 'perfect-checkout' ),
								'Perfect Checkout'
							),
							'desc'     => sprintf(
								/* translators: %s Perfect Checkout */
								esc_html__( 'Press "Activate %s" when ready to increase your conversion rates.', 'perfect-checkout' ),
								'Perfect Checkout'
							),
							'id'       => '',
							'type'     => 'pc_button',
							'value'    => sprintf(
								/* translators: %s Perfect Checkout */
								esc_html__( 'Activate %s', 'perfect-checkout' ),
								'Perfect Checkout'
							),
							'url'      => $switch_checkout_url,
							'autoload' => false,
						),
					);
				}
			} else {
				$connect_url = add_query_arg( array(
					'site' => get_site_url(),
				), PerfectCheckout::get_connect_url() );

				$new_settings = array(
					'setup_pc' => array(
						'title'    => sprintf(
							/* translators: %s Perfect Checkout */
							esc_html__( 'Setup %s', 'perfect-checkout' ),
							'Perfect Checkout'
						),
						'desc'     => sprintf(
							/* translators: %s Perfect Checkout */
							esc_html__( 'Go back to %s to complete the connection.', 'perfect-checkout' ),
							'Perfect Checkout'
						),
						'id'       => '',
						'type'     => 'pc_button',
						'value'    => sprintf(
							/* translators: %s Perfect Checkout */
							esc_html__( 'Connect %s', 'perfect-checkout' ),
							'Perfect Checkout'
						),
						'url'      => $connect_url,
						'autoload' => false,
					),
				);
			}

			array_splice( $settings, key( $myaccount_page ), 0, $new_settings );

			return $settings;
		} // END woocommerce_settings_pages()

		/**
		 * Add a post display state for "Perfect Checkout" page in the page list table.
		 *
		 * @access  public
		 * @since   1.0.2
		 * @version 1.5.0
		 * @param   array   $post_states An array of post display states.
		 * @param   WP_Post $post        The current post object.
		 */
		public function add_display_post_states( $post_states, $post ) {
			$perfect_checkout_page = absint( get_option( 'perfectcheckout_page_id' ) );

			if ( $post->ID == $perfect_checkout_page ) {
				$post_states['perfect_checkout_active'] = sprintf(
					/* translators: %s Perfect Checkout */
					esc_html__( '%s is Active', 'perfect-checkout' ),
					'Perfect Checkout'
				);
			}

			return $post_states;
		} // END add_display_post_states()

		/**
		 * Removes the "Edit" and "Trash" link from the page action links.
		 *
		 * @access public
		 * @since  1.5.0
		 * @param  array   $actions An array of page action links.
		 * @param  WP_Post $post   The current post object.
		 * @return array          An array of page action links.
		 */
		public function remove_page_action_links( $actions, $post ) {
			if ( $post->ID === absint( get_option( 'perfectcheckout_page_id' ) ) ) {
				unset( $actions['edit'] );
				unset( $actions['trash'] );
			}

			return $actions;
		} // END remove_page_action_links()

		/**
		 * Revokes a single WooCommerce API Key.
		 * 
		 * @access public
		 *
		 * @static
		 *
		 * @since 1.6.7
		 *
		 * @param string $consumer_key
		 * @param string $consumer_secret
		 *
		 * @return bool
		 */
		public static function revoke_wc_api_key( $consumer_key, $consumer_secret ) {
			global $wpdb;

			$delete = $wpdb->delete( $wpdb->prefix . 'woocommerce_api_keys', array( 'consumer_key' => wc_api_hash( $consumer_key ), 'consumer_secret' => $consumer_secret ), array( '%s', '%s' ) );

			return $delete;
		} // END revoke_wc_api_key()

		/**
		 * Revokes all WooCommerce API Keys associated with Perfect Checkout.
		 * 
		 * @access public
		 *
		 * @since 1.6.7
		 *
		 * @return bool
		 */
		public function revokes_all_pc_wc_api_keys() {
			global $wpdb;

			$table  = $wpdb->prefix . 'woocommerce_api_keys';
			$delete = $wpdb->query( "DELETE FROM $table WHERE description LIKE 'Perfect Checkout\ %';" );

			return $delete;
		} // END revokes_all_pc_wc_api_keys()

	} // END class

} // END if class exists

return new Perfect_Checkout_Admin();
