<?php
/**
 * Perfect Checkout - Connect store controller.
 *
 * Handles the request to connect or disconnect the store at Perfect Checkout.
 *
 * @author  Sébastien Dumont
 * @package Perfect Checkout\API
 * @since   1.3.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * REST API Connect store controller class.
 *
 * @package Perfect Checkout\API
 */
class Perfect_Checkout_API_Connect_Store_Controller {

	/**
	 * Endpoint namespace.
	 *
	 * @var string
	 */
	protected $namespace = 'pc/connect';

	/**
	 * Route base.
	 *
	 * @var string
	 */
	protected $rest_base = 'store';

	/**
	 * Register routes.
	 *
	 * @access public
	 */
	public function register_routes() {
		register_rest_route(
			$this->namespace,
			'/' . $this->rest_base,
			array(
				array(
					'methods'             => WP_REST_Server::READABLE,
					'callback'            => array( $this, 'disconnect_callback' ),
					'permission_callback' => array( $this, 'get_items_permissions_check' ),
				),
				array(
					'methods'             => WP_REST_Server::EDITABLE,
					'callback'            => array( $this, 'connect_callback' ),
					'permission_callback' => array( $this, 'update_items_permissions_check' ),
					'args'                => array(
						'license' => array(
							'required'          => true,
							'description'       => __( 'Unique license key.', 'perfect-checkout' ),
							'type'              => 'string',
							'sanitize_callback' => 'sanitize_text_field',
							'validate_callback' => 'rest_validate_request_arg',
						),
					),
				)
			),
		);
	} // register_routes()

	/**
	 * Check whether a given request has permission to read info.
	 *
	 * @access public
	 *
	 * @param WP_REST_Request $request Full details about the request.
	 *
	 * @return WP_Error|boolean
	 */
	public function get_items_permissions_check( $request ) {
		if ( ! wc_rest_check_user_permissions( 'read' ) ) {
			return new WP_Error( 'perfectcheckout_rest_cannot_view', __( 'Sorry, you cannot list resources.', 'perfect-checkout' ), array( 'status' => rest_authorization_required_code() ) );
		}

		return true;
	} // END get_items_permissions_check()

	/**
	 * Need write/create permission.
	 *
	 * @access public
	 *
	 * @param WP_REST_Request $request Full details about the request.
	 *
	 * @return WP_Error|boolean
	 */
	public function update_items_permissions_check( $request ) {
		if ( ! wc_rest_check_user_permissions( 'create' ) ) {
			return new WP_Error( 'perfectcheckout_rest_cannot_create', __( 'Sorry, you are not allowed to create resources.', 'perfect-checkout' ), array( 'status' => rest_authorization_required_code() ) );
		}

		return true;
	} // END update_items_permissions_check()

	/**
	 * Revokes access when disconnecting Perfect Checkout account.
	 *
	 * @access public
	 *
	 * @param WP_REST_Request $request Full details about the request.
	 *
	 * @return WP_REST_Response
	 */
	public function disconnect_callback( $request ) {
		// Delete license key.
		delete_option( 'perfectcheckout_license' );

		$consumer_key    = '';
		$consumer_secret = '';

		// Look up authorization header and check it's a valid.
		if ( isset( $_SERVER['HTTP_AUTHORIZATION'] ) && 0 === stripos( $_SERVER['HTTP_AUTHORIZATION'], 'basic ' ) ) {
			$exploded = explode( ':', base64_decode( substr( sanitize_text_field( wp_unslash( $_SERVER['HTTP_AUTHORIZATION'] ) ), 6 ) ), 2 ); // phpcs:ignore WordPress.Security.NonceVerification.Recommended

			// If valid return consumer key and consumer secret.
			if ( 2 == \count( $exploded ) ) {
				list( $consumer_key, $consumer_secret ) = $exploded;
			}
		}

		// If the $_GET parameters are present, use those.
		elseif ( ! empty( $_GET['consumer_key'] ) && ! empty( $_GET['consumer_secret'] ) ) { // WPCS: CSRF ok.
			$consumer_key    = $_GET['consumer_key']; // WPCS: CSRF ok, sanitization ok.
			$consumer_secret = $_GET['consumer_secret']; // WPCS: CSRF ok, sanitization ok.
		}

		// Remove WC API Keys.
		include_once dirname( PERFECTCHECKOUT_FILE ) . '/includes/admin/class-perfect-checkout-admin.php';
		Perfect_Checkout_Admin::revoke_wc_api_key( $consumer_key, $consumer_secret );

		// Response.
		$response = rest_ensure_response( 'disconnected' );
		$response->set_status( 200 );

		return $response;
	} // END disconnect_callback()

	/**
	 * Returns settings of Perfect Checkout.
	 *
	 * @access public
	 *
	 * @param WP_REST_Request $request Full details about the request.
	 *
	 * @return WP_REST_Response
	 */
	public function connect_callback( $request ) {
		/**
		 * Get and sanitize license.
		 */
		$license = ! empty( $request['license'] ) ? sanitize_text_field( $request['license'] ) : false;

		/**
		 * Check a license was provided.
		*/
		if ( empty( $license ) ) {
			return new WP_Error( 'perfectcheckout_no_license_found', __( 'No license found. Please provide a license key.', 'perfect-checkout' ), array( 'status' => 404 ) );
		}

		/**
		 * Update license.
		 */
		update_option( 'perfectcheckout_license', $license );

		/**
		 * Prepare response.
		 */
		$data = array(
			'updated' => true,
			'license' => get_option( 'perfectcheckout_license' ),
		);

		// Response.
		$response = rest_ensure_response( $data );
		$response->set_status( 200 );

		return $response;
	} // END connect_callback()

} // END class
