<?php
/**
 * Perfect Checkout - Perfect Checkout Dashboard Support.
 *
 * @author  Sébastien Dumont
 * @package Perfect Checkout\Dashboard
 * @since   1.6.5 Introduced.
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'Perfect_Checkout_Dashboard' ) ) {

	class Perfect_Checkout_Dashboard {

		/**
		 * Initialize Dashboard.
		 *
		 * @access public
		 *
		 * @static
		 */
		public static function init() {
			// Allows a featured image for a product to upload via base64 data via WC REST API.
			add_filter( 'woocommerce_rest_insert_product_object', array( __CLASS__, 'wc_rest_upload_featured_product_image_base64' ), 10, 2 );

			// Deletes all images attached to the product via WC REST API.
			add_action( 'woocommerce_rest_delete_product_object', array( __CLASS__, 'wc_rest_delete_product_images' ), 10, 3 );

			// Enhances the product response for product images via WC REST API.
			add_filter( 'woocommerce_rest_prepare_product_object', array( __CLASS__, 'wc_rest_get_product_images' ), 10, 2 );
		} // END init()

		/**
		 * Allows a featured image for a product to upload via base64 data.
		 *
		 * Both on creating a product and updating a product.
		 *
		 * @access public
		 *
		 * @static
		 *
		 * @param WC_Product      $product Product data.
		 * @param WP_REST_Request $request Request object.
		 *
		 * @return WC_Product $product Product data.
		 */
		public static function wc_rest_upload_featured_product_image_base64( $product, $request ) {
			$params = method_exists( $request, 'get_params' ) ? $request->get_params() : array();

			// Base 64 string passed?
			$featured_image_base64 = isset( $params['featured_image_base64'] ) ? $params['featured_image_base64'] : '';

			if ( ! empty( $featured_image_base64 ) ) {
				$time = time();

				// Product ID
				$product_id = $product->get_id();

				// Upload directory.
				$uploads = wp_upload_dir( $time );

				// Prepare file data
				$file_data = explode( '/', $featured_image_base64 );
				$extension = str_replace( 'base64', '', str_replace( ';', '', substr( $file_data[1], 0, 3 ) ) ); // Get the first 3 characters as the extension.
				$file_name = $product_id . '_featured_image.' . $extension; // Filename to save the image as.

				// Make file name unique.
				$file_name = wp_unique_filename( $uploads['path'], $file_name );

				// Strip the query strings from the file name.
				$file_name = str_replace( '?', '-', $file_name );
				$file_name = str_replace( '&', '-', $file_name );

				$file_name = self::wc_rest_unique_filename( $file_name, "." . $extension );

				// File location where the file will be saved.
				$file_location = $uploads['path'] . "/" . $file_name;

				// Create a new file in the WordPress uploads folder.
				$file = fopen( $file_location, 'wb' ); 

				// Get the image data.
				$data = explode( ',', $featured_image_base64 );

				// Save file data.
				fwrite( $file, base64_decode( $data[1] ) );

				// Clean up the file.
				fclose( $file ); 

				include_once ABSPATH . 'wp-admin/includes/media.php';

				if ( ! function_exists( 'wp_generate_attachment_metadata' ) ) {
					include( ABSPATH . 'wp-admin/includes/image.php' );
				}

				$info    = wp_check_filetype( $file_location );
				$title   = '';
				$content = '';

				if ( $image_meta = @wp_read_image_metadata( $file_location ) ) {
					if ( trim( $image_meta['title'] ) && ! is_numeric( sanitize_title( $image_meta['title'] ) ) ) {
						$title = wc_clean( $image_meta['title'] );
					}
					if ( trim( $image_meta['caption'] ) ) {
						$content = wc_clean( $image_meta['caption'] );
					}
				}

				// Prepare an array of post data for the attachment.
				$attachment = array(
					'guid'           => $uploads['url'] . "/" . $file_name,
					'post_mime_type' => $info['type'],
					'post_title'     => $title ? $title : basename( $file_location ),
					'post_content'   => $content
				);

				// Insert as attachment.
				$attachment_id = wp_insert_attachment( $attachment, $file_location, $product_id );

				if ( ! is_wp_error( $attachment_id ) ) {
					$meta_data = wp_generate_attachment_metadata( $attachment_id, $file_location );

					if ( ! is_wp_error( $meta_data ) ) {
						// Generate images.
						wp_update_attachment_metadata( $attachment_id, $meta_data );

						// Set featured image.
						update_post_meta( $product->get_id(), '_thumbnail_id', $attachment_id ); // Force the thumbnail ID meta to attach uploaded image.
						$product->set_image_id( $attachment_id );
					}
				}
			}

			return $product;
		} // END wc_rest_upload_featured_product_image_base64()

		/**
		 * Deletes all images attached to the product.
		 *
		 * If force is set, the images will permanently delete.
		 *
		 * @access public
		 *
		 * @static
		 *
		 * @param WC_Product       $product  Product data.
		 * @param WP_REST_Response $response Request response object.
		 * @param WP_REST_Request  $request  Full details about the request.
		 */
		public static function wc_rest_delete_product_images( $product, $response, $request ) {
			$force = (bool) $request['force'];

			$image_ids = array_merge( array( $product->get_image_id( 'view' ) ), $product->get_gallery_image_ids( 'view' ) );

			foreach ( $image_ids as $id ) {
				wp_delete_attachment( $id, $force );
			}
		} // END wc_rest_delete_product_images()

		/**
		 * Get the images for a product or product variation
		 * and returns all image sizes.
		 *
		 * @access public
		 *
		 * @static
		 *
		 * @param WP_REST_Response                $response Request response object.
		 * @param WC_Product|WC_Product_Variation $product Product instance.
		 *
		 * @return WP_REST_Response $response Request response object.
		 */
		public static function wc_rest_get_product_images( $response, $product ) {
			$images           = array();
			$attachment_ids   = array();
			$attachment_sizes = array_merge( get_intermediate_image_sizes(), array( 'full', 'custom' ) );

			// Add featured image.
			if ( $product->get_image_id() ) {
				$attachment_ids[] = $product->get_image_id();
			}

			// Add gallery images.
			$attachment_ids = array_merge( $attachment_ids, $product->get_gallery_image_ids() );

			$attachments = array();

			// Build image data.
			foreach ( $attachment_ids as $position => $attachment_id ) {
				$attachment_post = get_post( $attachment_id );
				if ( is_null( $attachment_post ) ) {
					continue;
				}

				// Get each image size of the attachment.
				foreach ( $attachment_sizes as $size ) {
					$image_src = ! empty( wp_get_attachment_image_src( $attachment_id, $size ) ) ? wp_get_attachment_image_src( $attachment_id, $size ) : array();
					$attachments[ $size ] = current( $image_src );
				}

				$featured = $position === 0 ? true : false; // phpcs:ignore WordPress.PHP.YodaConditions.NotYoda

				$images[] = array(
					'id'                => (int) $attachment_id,
					'date_created'      => wc_rest_prepare_date_response( $attachment_post->post_date, false ),
					'date_created_gmt'  => wc_rest_prepare_date_response( strtotime( $attachment_post->post_date_gmt ) ),
					'date_modified'     => wc_rest_prepare_date_response( $attachment_post->post_modified, false ),
					'date_modified_gmt' => wc_rest_prepare_date_response( strtotime( $attachment_post->post_modified_gmt ) ),
					'src'               => $attachments,
					'name'              => get_the_title( $attachment_id ),
					'alt'               => get_post_meta( $attachment_id, '_wp_attachment_image_alt', true ),
					'position'          => (int) $position,
					'featured'          => $featured,
				);
			}

			// Set a placeholder image if the product has no images set.
			if ( empty( $images ) ) {
				// Get each image size of the attachment.
				foreach ( $attachment_sizes as $size ) {
					$image_src = ! empty( wp_get_attachment_image_src( get_option( 'woocommerce_placeholder_image', 0 ), $size ) ) ? wp_get_attachment_image_src( get_option( 'woocommerce_placeholder_image', 0 ), $size ) : array();
					$attachments[ $size ] = current( $image_src );
				}

				$images[] = array(
					'id'       => 0,
					'date_created'      => wc_rest_prepare_date_response( current_time( 'mysql' ), false ), // Default to now.
					'date_created_gmt'  => wc_rest_prepare_date_response( time() ), // Default to now.
					'date_modified'     => wc_rest_prepare_date_response( current_time( 'mysql' ), false ),
					'date_modified_gmt' => wc_rest_prepare_date_response( time() ),
					'src'               => $attachments,
					'name'              => __( 'Placeholder', 'perfect-checkout' ),
					'alt'               => __( 'Placeholder', 'perfect-checkout' ),
					'position'          => 0,
					'featured'          => true,
				);
			}

			$response->data['images'] = $images;

			return $response;
		} // END wc_rest_get_product_images()

		/**
		 * Change filename to append random text.
		 *
		 * @access public
		 *
		 * @static
		 *
		 * @param string $full_filename Original filename with extension.
		 * @param string $ext           File Extension.
		 *
		 * @return string Modified filename.
		 */
		public static function wc_rest_unique_filename( $full_filename, $ext ) {
			$ideal_random_char_length = 6;   // Not going with a larger length because then downloaded filename will not be pretty.
			$max_filename_length      = 255; // Max file name length for most file systems.
			$length_to_prepend        = min( $ideal_random_char_length, $max_filename_length - strlen( $full_filename ) - 1 );

			if ( 1 > $length_to_prepend ) {
				return $full_filename;
			}

			$suffix   = strtolower( wp_generate_password( $length_to_prepend, false, false ) );
			$filename = $full_filename;

			if ( strlen( $ext ) > 0 ) {
				$filename = substr( $filename, 0, strlen( $filename ) - strlen( $ext ) );
			}

			$full_filename = str_replace(
				$filename,
				"$filename-$suffix",
				$full_filename
			);

			return $full_filename;
		} // END wc_rest_unique_filename()

	} // END class

	Perfect_Checkout_Dashboard::init();

} // END class exists
