<?php
/**
 * Perfect Checkout helpers.
 *
 * Provides functions that provide helpful data for the plugin.
 *
 * @author  Sébastien Dumont
 * @package Perfect Checkout\Helpers
 * @since   1.0.0
 * @version 1.2.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Perfect Checkout helper class.
 *
 * @package Perfect Checkout/Helpers
 */
class Perfect_Checkout_Helpers {

	/**
	 * Cache 'gte' comparison results for WooCommerce version.
	 *
	 * @var array
	 */
	private static $is_wc_version_gte = array();

	/**
	 * Cache 'gt' comparison results for WooCommerce version.
	 *
	 * @var array
	 */
	private static $is_wc_version_gt = array();

	/**
	 * Cache 'lte' comparison results for WooCommerce version.
	 *
	 * @var array
	 */
	private static $is_wc_version_lte = array();

	/**
	 * Cache 'lt' comparison results for WooCommerce version.
	 *
	 * @var array
	 */
	private static $is_wc_version_lt = array();

	/**
	 * Cache 'gt' comparison results for WP version.
	 *
	 * @var array
	 */
	private static $is_wp_version_gt = array();

	/**
	 * Cache 'gte' comparison results for WP version.
	 *
	 * @var array
	 */
	private static $is_wp_version_gte = array();

	/**
	 * Cache 'lt' comparison results for WP version.
	 *
	 * @var array
	 */
	private static $is_wp_version_lt = array();

	/**
	 * Cache 'lte' comparison results for CoCart version.
	 *
	 * @var array
	 */
	private static $is_cocart_version_lte = array();

	/**
	 * Cache 'lte' comparison results for CoCart Pro version.
	 *
	 * @var array
	 */
	private static $is_cocart_pro_version_lte = array();

	/**
	 * Helper method to get the version of the currently installed WooCommerce.
	 *
	 * @access private
	 * @return string
	 */
	private static function get_wc_version() {
		return defined( 'WC_VERSION' ) && WC_VERSION ? WC_VERSION : null;
	} // END get_wc_version()

	/**
	 * Returns true if the installed version of WooCommerce is 4.0 or greater.
	 *
	 * @access public
	 * @return boolean
	 */
	public static function is_wc_version_gte_4_0() {
		return self::is_wc_version_gte( '4.0' );
	} // END is_wc_version_gte_4_0()

	/**
	 * Returns true if the installed version of WooCommerce is 4.5 or greater.
	 *
	 * @access public
	 * @return boolean
	 */
	public static function is_wc_version_gte_4_5() {
		return self::is_wc_version_gte( '4.5' );
	} // END is_wc_version_gte_4_5()

	/**
	 * Returns true if the installed version of WooCommerce is lower than 4.5.
	 *
	 * @access public
	 * @return boolean
	 */
	public static function is_wc_version_lt_4_5() {
		return self::is_wc_version_lt( '4.5' );
	} // END is_wc_version_lt_4_5()

	/**
	 * Returns true if the installed version of WooCommerce is 4.8 or greater.
	 *
	 * @access public
	 * @return boolean
	 */
	public static function is_wc_version_gte_4_8() {
		return self::is_wc_version_gte( '4.8' );
	} // END is_wc_version_gte_4_5()

	/**
	 * Returns true if the installed version of WooCommerce is greater than or equal to $version.
	 *
	 * @access public
	 * @param  string $version the version to compare.
	 * @return boolean true if the installed version of WooCommerce is >= $version
	 */
	public static function is_wc_version_gte( $version ) {
		if ( ! isset( self::$is_wc_version_gte[ $version ] ) ) {
			self::$is_wc_version_gte[ $version ] = self::get_wc_version() && version_compare( self::get_wc_version(), $version, '>=' );
		}
		return self::$is_wc_version_gte[ $version ];
	} // END is_wc_version_gte()

	/**
	 * Returns true if the installed version of WooCommerce is greater than $version.
	 *
	 * @access public
	 * @param  string $version the version to compare.
	 * @return boolean true if the installed version of WooCommerce is > $version
	 */
	public static function is_wc_version_gt( $version ) {
		if ( ! isset( self::$is_wc_version_gt[ $version ] ) ) {
			self::$is_wc_version_gt[ $version ] = self::get_wc_version() && version_compare( self::get_wc_version(), $version, '>' );
		}

		return self::$is_wc_version_gt[ $version ];
	} // END is_wc_version_gt()

	/**
	 * Returns true if the installed version of WooCommerce is lower than or equal to $version.
	 *
	 * @access public
	 * @param  string $version the version to compare.
	 * @return boolean true if the installed version of WooCommerce is <= $version
	 */
	public static function is_wc_version_lte( $version ) {
		if ( ! isset( self::$is_wc_version_lte[ $version ] ) ) {
			self::$is_wc_version_lte[ $version ] = self::get_wc_version() && version_compare( self::get_wc_version(), $version, '<=' );
		}
		return self::$is_wc_version_lte[ $version ];
	} // END is_wc_version_lte()

	/**
	 * Returns true if the installed version of WooCommerce is less than $version.
	 *
	 * @access public
	 * @param  string $version The version to compare.
	 * @return boolean true if the installed version of WooCommerce is < $version
	 */
	public static function is_wc_version_lt( $version ) {
		if ( ! isset( self::$is_wc_version_lt[ $version ] ) ) {
			self::$is_wc_version_lt[ $version ] = self::get_wc_version() && version_compare( self::get_wc_version(), $version, '<' );
		}

		return self::$is_wc_version_lt[ $version ];
	} // END is_wc_version_lt()

	/**
	 * Returns true if the WooCommerce version does not meet Perfect Checkout requirements.
	 *
	 * @access public
	 * @static
	 * @return boolean
	 */
	public static function is_not_wc_version_required() {
		if ( version_compare( self::get_wc_version(), PerfectCheckout::$required_woo, '<' ) ) {
			return true;
		}

		return false;
	} // END is_note_wc_version_required()

	/**
	 * Returns true if the installed version of WordPress is greater than $version.
	 *
	 * @access public
	 * @param  string $version The version to compare.
	 * @return boolean
	 */
	public static function is_wp_version_gt( $version ) {
		if ( ! isset( self::$is_wp_version_gt[ $version ] ) ) {
			global $wp_version;

			self::$is_wp_version_gt[ $version ] = $wp_version && version_compare( $wp_version, $version, '>' );
		}

		return self::$is_wp_version_gt[ $version ];
	} // END is_wp_version_gt()

	/**
	 * Returns true if the installed version of WordPress is greater than or equal to $version.
	 *
	 * @access public
	 * @param  string $version The version to compare.
	 * @return boolean
	 */
	public static function is_wp_version_gte( $version ) {
		if ( ! isset( self::$is_wp_version_gte[ $version ] ) ) {
			global $wp_version;

			self::$is_wp_version_gte[ $version ] = $wp_version && version_compare( $wp_version, $version, '>=' );
		}

		return self::$is_wp_version_gte[ $version ];
	} // END is_wp_version_gte()

	/**
	 * Returns true if the installed version of WordPress is less than $version.
	 *
	 * @access public
	 * @param  string $version The version to compare.
	 * @return boolean
	 */
	public static function is_wp_version_lt( $version ) {
		if ( ! isset( self::$is_wp_version_lt[ $version ] ) ) {
			global $wp_version;

			self::$is_wp_version_lt[ $version ] = $wp_version && version_compare( $wp_version, $version, '<' );
		}

		return self::$is_wp_version_lt[ $version ];
	} // END is_wp_version_lt()

	/**
	 * These are the only screens Perfect Checkout will focus
	 * on displaying notices or enqueue scripts/styles.
	 *
	 * @access public
	 * @static
	 * @return array
	 */
	public static function perfect_checkout_get_admin_screens() {
		return array(
			'dashboard',
			'plugins',
			'toplevel_page_perfect-checkout',
		);
	} // END perfect_checkout_get_admin_screens()

	/**
	 * Returns true|false if the user is on a Perfect Checkout page.
	 *
	 * @access public
	 * @static
	 * @return bool
	 */
	public static function is_perfect_checkout_admin_page() {
		$screen    = get_current_screen();
		$screen_id = $screen ? $screen->id : '';

		if ( ! in_array( $screen_id, self::perfect_checkout_get_admin_screens() ) ) {
			return false;
		}

		return true;
	} // END is_perfect_checkout_admin_page()

	/**
	 * Checks if the current user has the capabilities to install a plugin.
	 *
	 * @access public
	 * @static
	 * @since  1.0.7
	 * @return bool
	 */
	public static function user_has_capabilities() {
		if ( current_user_can( apply_filters( 'perfect_checkout_install_capability', 'install_plugins' ) ) ) {
			return true;
		}

		// If the current user can not install plugins then return nothing!
		return false;
	} // END user_has_capabilities()

	/**
	 * Get current admin page URL.
	 *
	 * Returns an empty string if it cannot generate a URL.
	 *
	 * @access public
	 * @static
	 * @return string
	 */
	public static function perfect_checkout_get_current_admin_url() {
		$uri = isset( $_SERVER['REQUEST_URI'] ) ? esc_url_raw( wp_unslash( $_SERVER['REQUEST_URI'] ) ) : '';
		$uri = preg_replace( '|^.*/wp-admin/|i', '', $uri );

		if ( ! $uri ) {
			return '';
		}

		return admin_url( $uri );
	} // END perfect_checkout_get_current_admin_url()

	/**
	 * Checks if CoCart Lite is installed.
	 *
	 * @access public
	 * @static
	 * @return array
	 */
	public static function is_cocart_installed() {
		$active_plugins = (array) get_option( 'active_plugins', array() );

		if ( is_multisite() ) {
			$active_plugins = array_merge( $active_plugins, get_option( 'active_sitewide_plugins', array() ) );
		}

		return in_array( 'cart-rest-api-for-woocommerce/cart-rest-api-for-woocommerce.php', $active_plugins ) || array_key_exists( 'cart-rest-api-for-woocommerce/cart-rest-api-for-woocommerce.php', $active_plugins );
	} // END is_cocart_installed()

	/**
	 * Checks if CoCart Pro is installed.
	 *
	 * @access public
	 * @static
	 * @return array
	 */
	public static function is_cocart_pro_installed() {
		$active_plugins = (array) get_option( 'active_plugins', array() );

		if ( is_multisite() ) {
			$active_plugins = array_merge( $active_plugins, get_option( 'active_sitewide_plugins', array() ) );
		}

		return in_array( 'cocart-pro/cocart-pro.php', $active_plugins ) || array_key_exists( 'cocart-pro/cocart-pro.php', $active_plugins );
	} // END is_cocart_pro_installed()

	/**
	 * Checks if the API package controller file exists.
	 *
	 * @access public
	 * @static
	 * @param  string $controller Controller file name.
	 * @return boolean
	 */
	public static function package_controller_exists( $controller ) {
		return file_exists( untrailingslashit( plugin_dir_path( PERFECTCHECKOUT_FILE ) ) . '/packages/' . $controller );
	} // END package_controller_exists()

	/**
	 * Returns true if the installed version of CoCart is lower than or equal to $version.
	 *
	 * @access public
	 * @param  string $version The version to compare.
	 * @return boolean true if the installed version of CoCart is <= $version
	 */
	public static function is_cocart_version_lte( $version ) {
		if ( ! isset( self::$is_cocart_version_lte[ $version ] ) ) {
			self::$is_cocart_version_lte[ $version ] = COCART_VERSION && version_compare( COCART_VERSION, $version, '<=' );
		}

		return self::$is_cocart_version_lte[ $version ];
	} // END is_cocart_version_lte()

	/**
	 * Returns true if the installed version of CoCart Pro is lower than or equal to $version.
	 *
	 * @access public
	 * @param  string $version The version to compare.
	 * @return boolean true if the installed version of CoCart Pro is <= $version
	 */
	public static function is_cocart_pro_version_lte( $version ) {
		if ( ! isset( self::$is_cocart_pro_version_lte[ $version ] ) ) {
			self::$is_cocart_pro_version_lte[ $version ] = COCART_PRO_VERSION && version_compare( COCART_PRO_VERSION, $version, '<=' );
		}

		return self::$is_cocart_pro_version_lte[ $version ];
	} // END is_cocart_pro_version_lte()

	/**
	 * Helper method to get the version of the currently installed Perfect Checkout.
	 *
	 * @access public
	 * @static
	 * @return string
	 */
	public static function get_perfectcheckout_version() {
		return defined( 'PERFECTCHECKOUT_VERSION' ) && ! empty( PERFECTCHECKOUT_VERSION ) ? PERFECTCHECKOUT_VERSION : null;
	} // END get_perfectcheckout_version()

	/**
	 * Check how long Perfect Checkout has been active for.
	 *
	 * @access public
	 * @since  1.2.0
	 * @static
	 * @param  int $seconds - Time in seconds to check.
	 * @return bool Whether or not WooCommerce admin has been active for $seconds.
	 */
	public static function perfectcheckout_active_for( $seconds = '' ) {
		if ( empty( $seconds ) ) {
			return true;
		}

		// Getting install timestamp.
		$perfectcheckout_installed = get_option( 'perfectcheckout_install_date', false );

		if ( false === $perfectcheckout_installed ) {
			return false;
		}

		return ( ( time() - $perfectcheckout_installed ) >= $seconds );
	} // END perfectcheckout_active_for()

	/**
	 * Collects the additional data necessary.
	 *
	 * @access public
	 * @static
	 * @since  1.2.0
	 * @return array The shortlink data.
	 */
	public static function collect_additional_data() {
		$memory = WP_MEMORY_LIMIT;

		if ( function_exists( 'wc_let_to_num' ) ) {
			$memory = wc_let_to_num( $memory );
		}

		if ( function_exists( 'memory_get_usage' ) ) {
			$system_memory = @ini_get( 'memory_limit' ); // phpcs:ignore WordPress.PHP.NoSilencedErrors.Discouraged

			if ( function_exists( 'wc_let_to_num' ) ) {
				$system_memory = wc_let_to_num( $system_memory );
			}

			$memory = max( $memory, $system_memory );
		}

		// WordPress 5.5+ environment type specification.
		// 'production' is the default in WP, thus using it as a default here, too.
		$environment_type = 'production';
		if ( function_exists( 'wp_get_environment_type' ) ) {
			$environment_type = wp_get_environment_type();
		}

		return array(
			'php_version'             => self::get_php_version(),
			'wp_version'              => self::get_wordpress_version(),
			'wc_version'              => self::get_wc_version(),
			'perfectcheckout_version' => self::get_perfectcheckout_version(),
			'days_active'             => self::get_days_active(),
			'debug_mode'              => ( defined( 'WP_DEBUG' ) && WP_DEBUG ) ? 'Yes' : 'No',
			'sandbox_mode'            => ( defined( 'PERFECTCHECKOUT_DEBUG' ) ) ? PERFECTCHECKOUT_DEBUG : 'No',
			'memory_limit'            => size_format( $memory ),
			'user_language'           => self::get_user_language(),
			'multisite'               => is_multisite() ? 'Yes' : 'No',
			'environment_type'        => $environment_type,
		);
	} // END collect_additional_data()

	/**
	 * Gets the current site's PHP version, without the extra info.
	 *
	 * @access private
	 * @static
	 * @since  1.2.0
	 * @return string The PHP version.
	 */
	private static function get_php_version() {
		$version = explode( '.', PHP_VERSION );

		return (int) $version[0] . '.' . (int) $version[1];
	} // END get_php_version()

	/**
	 * Gets the current site's WordPress version.
	 *
	 * @access protected
	 * @static
	 * @since  1.2.0
	 * @return string The wp_version.
	 */
	protected static function get_wordpress_version() {
		return $GLOBALS['wp_version'];
	} // END get_wordpress_version()

	/**
	 * Gets the number of days the plugin has been active.
	 *
	 * @access private
	 * @static
	 * @since  1.2.0
	 * @return int The number of days the plugin is active.
	 */
	private static function get_days_active() {
		$date_activated = get_option( 'perfectcheckout_install_date', time() );
		$datediff       = ( time() - $date_activated );
		$days           = (int) round( $datediff / DAY_IN_SECONDS );

		return $days;
	} // END get_days_active()

	/**
	 * Gets the user's language.
	 *
	 * @access private
	 * @static
	 * @since  1.2.0
	 * @return string The user's language.
	 */
	private static function get_user_language() {
		if ( function_exists( 'get_user_locale' ) ) {
			return get_user_locale();
		}

		return false;
	} // END get_user_language()

} // END class

return new Perfect_Checkout_Helpers();
